@@ -1,5 +1,5 @@
 /*
- * * Copyright (C) 2009 Ali <aliov@xfce.org>
+ * * Copyright (C) 2009-2011 Ali <aliov@xfce.org>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -34,6 +34,9 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
+#ifdef XFCE_DISABLE_DEPRECATED
+#undef XFCE_DISABLE_DEPRECATED
+#endif
 #include <libxfce4util/libxfce4util.h>
 #include <libxfcegui4/libxfcegui4.h>
 
@@ -70,8 +73,13 @@
 /*
  * DBus Glib init
  */
-static void parole_player_dbus_class_init (ParolePlayerClass *klass);
-static void parole_player_dbus_init       (ParolePlayer *player);
+static void parole_player_dbus_class_init  (ParolePlayerClass *klass);
+static void parole_player_dbus_init        (ParolePlayer *player);
+
+static void parole_player_disc_selected_cb (ParoleDisc *disc, 
+					    const gchar *uri, 
+					    const gchar *device, 
+					    ParolePlayer *player);
 
 /*
  * GtkBuilder Callbacks
@@ -130,6 +138,12 @@ void            parole_player_menu_add_cb               (GtkWidget *widget,
 void            parole_player_menu_exit_cb              (GtkWidget *widget,
 							 ParolePlayer *player);
 
+void		dvd_iso_mi_activated_cb			(GtkWidget *widget,
+							 ParolePlayer *player);
+
+void		cd_iso_mi_activated_cb			(GtkWidget *widget,
+							 ParolePlayer *player);
+
 void            parole_player_volume_up 		(GtkWidget *widget, 
 							 ParolePlayer *player);
 
@@ -333,6 +347,87 @@ void parole_player_show_hide_playlist (GtkButton *button, ParolePlayer *player)
     g_object_unref (img);
 }
 
+typedef enum
+{
+    PAROLE_ISO_IMAGE_DVD,
+    PAROLE_ISO_IMAGE_CD
+} ParoleIsoImage;
+
+static void
+iso_files_folder_changed_cb (GtkFileChooser *widget, gpointer data)
+{
+    gchar *folder;
+    folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (widget));
+    
+    if ( folder )
+    {
+	parole_rc_write_entry_string ("iso-image-folder", PAROLE_RC_GROUP_GENERAL, folder);
+	g_free (folder);
+    }
+}
+
+static void
+parole_player_open_iso_image (ParolePlayer *player, ParoleIsoImage image)
+{
+    GtkWidget *chooser;
+    GtkFileFilter *filter;
+    gchar *file = NULL;
+    const gchar *folder;
+    gint response;
+    
+    chooser = gtk_file_chooser_dialog_new (_("Open ISO image"), GTK_WINDOW (player->priv->window),
+					   GTK_FILE_CHOOSER_ACTION_OPEN,
+					   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					   GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+					   NULL);
+				
+    gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (chooser), FALSE);
+    
+    folder = parole_rc_read_entry_string ("iso-image-folder", PAROLE_RC_GROUP_GENERAL, NULL);
+    
+    if ( folder )
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), folder);
+    
+    g_signal_connect (chooser, "current-folder-changed",
+		      G_CALLBACK (iso_files_folder_changed_cb), NULL);
+    
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name (filter, image == PAROLE_ISO_IMAGE_CD ? _("CD image") : _("DVD image"));
+    gtk_file_filter_add_mime_type (filter, "application/x-cd-image");
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (chooser), filter);
+
+    gtk_window_set_default_size (GTK_WINDOW (chooser), 680, 480);
+    response = gtk_dialog_run (GTK_DIALOG (chooser));
+    
+    if ( response == GTK_RESPONSE_OK )
+    {
+	file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+    }
+    
+    gtk_widget_destroy (chooser);
+    
+    if ( file )
+    {
+	gchar *uri;
+	//FIXME: vcd will word for svcd?
+	uri = g_strdup_printf ("%s%s", PAROLE_ISO_IMAGE_CD ? "dvd://" : ("vcd://"), file);
+	TRACE ("Playing ISO image %s", uri);
+	parole_player_disc_selected_cb (NULL, uri, NULL, player);
+	g_free (file);
+	g_free (uri);
+    }
+}
+
+void dvd_iso_mi_activated_cb (GtkWidget *widget, ParolePlayer *player)
+{
+    parole_player_open_iso_image (player, PAROLE_ISO_IMAGE_DVD);
+}
+
+void cd_iso_mi_activated_cb (GtkWidget *widget,	 ParolePlayer *player)
+{
+    parole_player_open_iso_image (player, PAROLE_ISO_IMAGE_CD);
+}
+
 static void
 parole_player_change_range_value (ParolePlayer *player, gdouble value)
 {
@@ -444,6 +539,21 @@ parole_player_media_progressed_cb (ParoleGst *gst, const ParoleStream *stream, g
     }
 }
 
+static void
+parole_player_seekable_notify (ParoleStream *stream, GParamSpec *spec, ParolePlayer *player)
+{
+    gboolean seekable;
+    
+    g_object_get (G_OBJECT (stream),
+		  "seekable", &seekable,
+		  NULL);
+		  
+    gtk_widget_set_tooltip_text (GTK_WIDGET (player->priv->range), seekable ? NULL : _("Media stream is not seekable"));
+    gtk_widget_set_sensitive (GTK_WIDGET (player->priv->range), seekable);
+    gtk_widget_set_sensitive (player->priv->seekf, seekable);
+    gtk_widget_set_sensitive (player->priv->seekb, seekable);
+}
+
 static void
 parole_player_set_playpause_button_image (GtkWidget *widget, const gchar *stock_id)
 {
@@ -516,8 +626,6 @@ parole_player_playing (ParolePlayer *player, const ParoleStream *stream)
     gboolean seekable;
     gboolean live;
     
-    player->priv->state = PAROLE_MEDIA_STATE_PLAYING;
-    
     pix = parole_icon_load ("player_play", 16);
     
     if ( !pix )
@@ -563,8 +671,6 @@ parole_player_paused (ParolePlayer *player)
 {
     GdkPixbuf *pix = NULL;
     
-    player->priv->state = PAROLE_MEDIA_STATE_PAUSED;
-    
     TRACE ("Player paused");
     
     pix = parole_icon_load (GTK_STOCK_MEDIA_PAUSE, 16);
@@ -596,8 +702,6 @@ parole_player_stopped (ParolePlayer *player)
 {
     TRACE ("Player stopped");
     
-    player->priv->state = PAROLE_MEDIA_STATE_STOPPED;
-    
     gtk_widget_set_sensitive (player->priv->play_pause, 
 			      parole_media_list_is_selected_row (player->priv->list) || 
 			      !parole_media_list_is_empty (player->priv->list));
@@ -728,7 +832,9 @@ parole_player_reset_saver_changed (ParolePlayer *player, const ParoleStream *str
 		      NULL);
 		      
 	if ( has_video )
+	{
 	    parole_screen_saver_inhibit (player->priv->screen_saver);
+	}
     }
     else
 	parole_screen_saver_uninhibit (player->priv->screen_saver);
@@ -739,6 +845,9 @@ parole_player_media_state_cb (ParoleGst *gst, const ParoleStream *stream, Parole
 {
     PAROLE_DEBUG_ENUM ("State callback", state, GST_ENUM_TYPE_MEDIA_STATE);
 
+
+    player->priv->state = state;
+    
     parole_player_reset_saver_changed (player, stream);
     
     if ( state == PAROLE_MEDIA_STATE_PLAYING )
@@ -971,7 +1080,7 @@ parole_player_move_fs_window (ParolePlayer *player)
 		     rect.height + rect.y - player->priv->play_box->allocation.height);
 }
 
-static void
+void
 parole_player_full_screen (ParolePlayer *player, gboolean fullscreen)
 {
     gint npages;
@@ -1899,6 +2008,9 @@ parole_player_init (ParolePlayer *player)
     gtk_widget_realize (player->priv->gst);
     gtk_widget_show (player->priv->gst);
 
+    g_signal_connect (G_OBJECT (parole_gst_get_stream (PAROLE_GST (player->priv->gst))), "notify::seekable",
+		      G_CALLBACK (parole_player_seekable_notify), player);
+
     parole_player_change_volume (player, 
 				 (gdouble) (parole_rc_read_entry_int ("volume", PAROLE_RC_GROUP_GENERAL, 100)/100.));
 
@@ -1969,7 +2081,19 @@ ParoleMediaList	*parole_player_get_media_list (ParolePlayer *player)
 
 void parole_player_play_uri_disc (ParolePlayer *player, const gchar *uri, const gchar *device)
 {
-    parole_player_disc_selected_cb (NULL, uri, device, player);
+    if ( uri )
+    {
+	parole_player_disc_selected_cb (NULL, uri, device, player);
+    }
+    else if (device)
+    {
+	gchar *uri_local = parole_get_uri_from_unix_device (device);
+	if ( uri_local )
+	{
+	    parole_player_disc_selected_cb (NULL, uri_local, device, player);
+	    g_free (uri_local);
+	}
+    }
 }
 
 void parole_player_terminate (ParolePlayer *player)
